from io import BytesIO
import streamlit as st
import pandas as pd
from PIL import Image
from templates import *


st.title("Chart bot")

if "messages" not in st.session_state:
    st.session_state.messages = []

for message in st.session_state.messages:
    with st.chat_message(message["role"]):
        if "image" in message:
            image_data = message["image"]
            image = Image.open(BytesIO(image_data))
            st.image(image, width=400)
        else:
            st.markdown(message["content"])

st.sidebar.title("Upload Options")
uploaded_file = st.sidebar.file_uploader("Upload CSV, Excel, or JSON file", type=["csv", "xlsx", "json"])

if uploaded_file:
    try:
        if uploaded_file.name.endswith(".csv"):
            df = pd.read_csv(uploaded_file)
        elif uploaded_file.name.endswith(".xlsx"):
            df = pd.read_excel(uploaded_file)
        elif uploaded_file.name.endswith(".json"):
            df = pd.read_json(uploaded_file)
        else:
            st.sidebar.error("Unsupported file format.")
        
        st.sidebar.success("File uploaded successfully!")
    except Exception as e:
        st.sidebar.error(f"Error loading file: {e}")
if prompt := st.chat_input("What is up?"):
    st.session_state.messages.append({"role": "user", "content": prompt})
    with st.chat_message("user"):
        st.markdown(prompt)
    retries = 3  
    success = False  
    for attempt in range(retries):
        try:
            with st.chat_message("assistant"):
                user_query = prompt
                output = understand_query(user_query)
                query_eval = eval(output[output.find('{'):output.rfind('}')+1])

                if query_eval['ifchart'] == 'yes':
                    query = f"""User_query: {user_query}
                                Expected chart type : {query_eval['type_of_chart']}"""
                    op_chart = fn_create_chart(query=query, columns=f"{df.columns}")
                    op_chart = op_chart[op_chart.find('['):op_chart.rfind(']')+1]
                    op_chart = eval(op_chart)
                    for i in op_chart:
                        exec(i)
                    fig = chart.get_figure()
                    buf = BytesIO()
                    fig.savefig(buf, format="PNG")
                    buf.seek(0)
                    image = Image.open(buf)
                    st.image(image, width=400)
                    # st.session_state.messages.append({"role": "assistant", "image": image})
                    st.session_state.messages.append({"role": "assistant", "image": buf.getvalue()})

                elif query_eval['ifdetails'] == 'yes':
                    op_sum = fn_create_summary(query=user_query, columns=f"{df.columns}")
                    op_sum = op_sum[op_sum.find('['):op_sum.rfind(']')+1]
                    op_sum = eval(op_sum)
                    for i in op_sum:
                        exec(i)
                    final_sum = fn_final_summary(f"User query: {user_query}.Data available: {output}")
                    st.write(final_sum)
                    st.session_state.messages.append({"role": "assistant", "content": final_sum})

                else:
                    fnl_op = fn_non_of_above(user_query)
                    st.write(fnl_op)
                    st.session_state.messages.append({"role": "assistant", "content": fnl_op})

                success = True 
                break 

        except Exception as e:
            pass

    if not success:
        st.session_state.messages.append({"role": "assistant", "content": "Error: Unable to process your request."})




from groq import Groq
from app import Analysischartbot

launcher = Analysischartbot()
client = Groq(api_key=launcher.get_groq_api_key())

def understand_query(query:str,model_:str = "Llama3-70b-8192"):
    try:
        template = f"""Your an helpful assistant to get understanding about the userquery based on the given conditions.
                    <userquery>{query}</userquery>. from the user query i want you to check the following conditions.
                    {{
                        "ifchart" : "if the user want to create a chart or not. value yes/no",
                        "type_of_chart" : "if the user want to create a chart,identify the type of chart user want to create eg (pie,bar,histogram) if no chart then 'None'.\
                                        Sometime user not explicitely tell which type of chart, for example,if user want to understand contribution (pie best option), likely understand which type of chart is suitable in that condition and return the same",
                        "ifdetails" : "Identify wheather the user asking for, understand the data through summary like , shape of the dataframe, null values or counts,value should yes or no.",
                        "type_of_details" : "which details user wants to understand, maybe dataframe summary, null values,shapes, understand acocrdingly and add the value,Put 'None' if ifdetails is no"
                        "None_of_above" : "if user query is not related to either charts or details about the data. value yes or no"
                    }}.output shoud be only the json with the same key as given in example, with the values understood from user query.
                    Output should only a json without any explanation.
                    """
        completion = client.chat.completions.create(
            model=model_,   
            messages=[
                {
                    "role": "user",
                    "content": template
                }
            ],
            temperature=0.9,
            max_tokens=1024,
        )
        return completion.choices[0].message.content
    except Exception as e:
        return f"Error occured: {str(e)}"

def fn_create_chart(query:str,columns:dict,model_:str = "Llama3-70b-8192"):
    try:
        completion = client.chat.completions.create(
            model=model_, 
            messages=[
                {
                    "role": "system",
                    "content": f"""your a helpful assistant to help user to process with the dataframe df.\
                        Your job is to create a python pandas code, to process the data based on user query.\
                        Data is already loaded into df (pandas Dataframe). column are {columns}.you should create a python code to create a chart using matplotlib or seaborn,
                        then each line of code you can create in an array like.include the import statements if you use matplotlib or seaborn.
                        Do not include plt.show() in the code.
                        [
                            'chart = df["chapterName"].value_counts().plot(kind="bar")',
                            'chart.set_xlabel("Chapter Name")',
                            'chart.set_ylabel("Count")',
                            'chart.set_title("Chapter Name Distribution")',
                        ]. the code need to execute should always in array, and if you generating any chart , give the variable name 'chart'.
                        """
                },
                {
                    "role": "user",
                    "content": query
                }
            ],
            temperature=0.9,
            max_tokens=1024,
        )
        return completion.choices[0].message.content
    except Exception as e:
        return f"Error occured: {str(e)}" 

def fn_create_summary(query:str,columns:dict,model_:str = "Llama3-70b-8192"):
    try:
        completion = client.chat.completions.create(
            model=model_,  
            messages=[
                {
                    "role": "system",
                    "content": f"""your a helpful assistant to help user to process with the dataframe df.\
                        Your job is to create a python pandas code, to process the data based on user query.\
                        Data is already loaded into df (pandas Dataframe). column are {columns}.you should create a python code to analyze the df based on the user query,
                        then each line of code you can create in an array like.Python code should create a variable 'output' which will have the results in '[(),(),...]' format (tuples inside array).
                        for an example if user asking about the null values in one column output variable should have [(column name,count of null values)].
                        then each line of code you can create in an array like, for an example if i want to calculate the missing value count in chapterName column
                        [
                            'missing_count = df['chapterName'].isna().sum()',
                            'output = [('Total missing value in chapterName is: ',missing_count)',..]'
                        ]. the code need to execute should always in array, and if you generating any output , give the variable name 'output'.
                        Do not include any explanation in output with the array.
                        """
                },
                {
                    "role": "user",
                    "content": query
                }
            ],
            temperature=0.9,
            max_tokens=1024,
        )
        return completion.choices[0].message.content
    except Exception as e:
        return f"Error occured: {str(e)}" 

def fn_final_summary(query:str,model_:str = "Llama3-70b-8192"):
    try:
        completion = client.chat.completions.create(
            model=model_,   
            messages=[
                {
                    "role": "system",
                    "content": f"""Your an helpful assistant to explain the answer to the user based on the \
                        Data available and the user query.Output should be just the short explanation, \
                        without giving input Data available in the response.
                        """
                },
                {
                    "role": "user",
                    "content": query
                }
            ],
            temperature=0.9,
            max_tokens=1024,
        )
        return completion.choices[0].message.content
    except Exception as e:
        return f"Error occured: {str(e)}" 

def fn_non_of_above(query:str,model_:str = "Llama3-70b-8192"):
    try:
        completion = client.chat.completions.create(
            model=model_,  
            messages=[
                {
                    "role": "system",
                    "content": f"""Your an helpful assistant to visualize and explain the data.Keep the response short as possible
                        """
                },
                {
                    "role": "user",
                    "content": query
                }
            ],
            temperature=0.9,
            max_tokens=1024,
        )
        return completion.choices[0].message.content
    except Exception as e:
        return f"Error occured: {str(e)}" 


import subprocess
import webbrowser
import time,os

class ZohencelmlBot:
    """
    A class to manage the Zohencelml bot and provide the Groq API key.
    """

    DEFAULT_GROQ_API_KEY = "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"  # Default API key
    DEFAULT_GROQ_MODEL = "llama3-70b-8192"
    def __init__(self, groq_api_key: str = None,groq_model: str = None):
        """
        Initializes the bot with the Groq API key. If not provided, uses the default API key.
        
        Parameters:
            groq_api_key (str, optional): The Groq API key (default is provided).
        """
        self.groq_api_key = groq_api_key or self.DEFAULT_GROQ_API_KEY
        self.groq_model = groq_model or self.DEFAULT_GROQ_MODEL
    def get_groq_api_key(self) -> str:
        """
        Returns the Groq API key.

        Returns:
            str: The Groq API key.
        """
        return self.groq_api_key
    def get_groq_model(self) -> str:
        return self.groq_model
    def run(self):
        """
        Launches the FastAPI app using `uvicorn main:app` command and opens the browser.
        """
        try:
            print("Launching FastAPI app using uvicorn...")
            
            # Start the FastAPI app in a subprocess
            process = subprocess.Popen(
                ["uvicorn", "zohencel_ai.mlbot.main:app"], cwd=os.path.dirname(__file__)
            )
            
            # Wait for a moment to allow the server to start
            time.sleep(3)
            
            # Open the browser with the specified URL
            url = "https://zohencelai.github.io/MLBot/"
            print(f"Opening browser at {url}...")
            webbrowser.open(url)
            
            # Wait for the subprocess to complete (if needed)
            process.wait()
        except subprocess.CalledProcessError as e:
            print(f"Failed to launch FastAPI app: {e}")
        except Exception as e:
            print(f"An unexpected error occurred: {e}")



import pandas as pd
import pickle
import os
import numpy as np
from train import *

current_dir = os.path.dirname(os.path.abspath(__file__))
pickle_file_path = os.path.join(current_dir, "pipeline_model.pkl")

with open(pickle_file_path, "rb") as f:
    pipeline = pickle.load(f)

def get_batch_predictions(test_df):
    test_df = test_df.dropna()
    test_df = test_df.drop_duplicates()
    predictions = pipeline.predict(test_df)
    test_df['predictions'] = list(predictions)
    return test_df